// src/installer/downloader.rs
use anyhow::{Result, Context, anyhow};
use reqwest::Client;
use tokio::io::AsyncWriteExt;
use std::path::{Path, PathBuf};
use std::sync::Arc;
use std::time::Duration;
use futures_util::StreamExt;
use tokio::sync::Mutex;
use std::fs::metadata;
use reqwest::header::{HeaderMap, HeaderValue, RANGE, USER_AGENT};
use log::{info, warn, error};
use tokio::fs::File;
use crate::app::{GameType};

pub struct DownloadManager {
    client: Client,
    progress_callback: Arc<Mutex<Box<dyn Fn(f32, &str) + Send + Sync>>>,
    temp_dir: PathBuf,
}

impl DownloadManager {
    pub fn new<F>(progress_callback: F) -> Self 
    where 
        F: Fn(f32, &str) + Send + Sync + 'static 
    {
        let client = Client::builder()
            .timeout(Duration::from_secs(1800))
            .build()
            .unwrap_or_default();
            
        let temp_dir = std::env::temp_dir().join("sinfar_installer");
        println!("Files will be downloaded to: {}", temp_dir.display());
        // Ensure temp directory exists
        if !temp_dir.exists() {
            if let Err(e) = std::fs::create_dir_all(&temp_dir) {
                warn!("Failed to create temp directory: {}", e);
            }
        }
        
        Self {
            client,
            progress_callback: Arc::new(Mutex::new(Box::new(progress_callback))),
            temp_dir,
        }
    }
    
    /// Downloads a file with resume capability and progress tracking
    pub async fn download_file(&self, url: &str, output_path: &Path) -> Result<()> {
        info!("Starting download of {} to {}", url, output_path.display());
        
        // Create parent directories if they don't exist
        if let Some(parent) = output_path.parent() {
            if !parent.exists() {
                tokio::fs::create_dir_all(parent).await?;
            }
        }
        
        // Check if we can resume a previous download
        let mut headers = HeaderMap::new();
        headers.insert(USER_AGENT, HeaderValue::from_static("Sinfar NWN Installer"));
        
        let mut downloaded_size: u64 = 0;
        
        if output_path.exists() {
            if let Ok(metadata) = tokio::fs::metadata(output_path).await {
                downloaded_size = metadata.len();
                if downloaded_size > 0 {
                    info!("Resuming download from byte {}", downloaded_size);
                    headers.insert(RANGE, HeaderValue::from_str(&format!("bytes={}-", downloaded_size))?);
                }
            }
        }
        
        let response = self.client.get(url)
            .headers(headers)
            .send()
            .await
            .context("Failed to send HTTP request")?;
            
        // Check if the server supports resuming
        let can_resume = response.status().is_success() && response.status().as_u16() == 206;
        let status = response.status();
        
        if !status.is_success() {
            return Err(anyhow!("Server returned error status: {}", status));
        }
        
        // Get total file size
        let content_length = if can_resume {
            if let Some(content_range) = response.headers().get("content-range") {
                let range_str = content_range.to_str().unwrap_or("");
                if let Some(size_part) = range_str.split('/').nth(1) {
                    size_part.parse::<u64>().unwrap_or(0)
                } else {
                    response.content_length().unwrap_or(0) + downloaded_size
                }
            } else {
                response.content_length().unwrap_or(0) + downloaded_size
            }
        } else {
            response.content_length().unwrap_or(0)
        };
        
        info!("Download size: {} bytes", content_length);
        
        // Open file in appropriate mode (append if resuming, create/truncate if starting fresh)
        let mut file = if can_resume && downloaded_size > 0 {
            info!("Opening file for append");
            tokio::fs::OpenOptions::new()
                .write(true)
                .append(true)
                .open(output_path)
                .await?
        } else {
            info!("Creating new file");
            downloaded_size = 0;
            tokio::fs::File::create(output_path).await?
        };
        
        // Stream the response
        let mut stream = response.bytes_stream();
        let progress_callback = self.progress_callback.clone();
        let filename = output_path.file_name()
            .and_then(|f| f.to_str())
            .unwrap_or("file");
            
        let mut last_update = std::time::Instant::now();
        
        while let Some(chunk_result) = stream.next().await {
            let chunk = chunk_result?;
            file.write_all(&chunk).await?;
            
            downloaded_size += chunk.len() as u64;
            
            // Only update progress every 100ms to avoid overwhelming the UI
            let now = std::time::Instant::now();
            if now.duration_since(last_update) > Duration::from_millis(100) || content_length == downloaded_size {
                if content_length > 0 {
                    let progress = downloaded_size as f32 / content_length as f32;
                    let callback = progress_callback.lock().await;
                    callback(progress, filename);
                }
                last_update = now;
            }
        }
        
        info!("Download completed: {}", output_path.display());
        Ok(())
    }
    
    /// Downloads all required files for the installation
    pub async fn download_all_files(&self, game_type: &GameType) -> Result<Vec<PathBuf>> {
        let mut downloaded_files = Vec::new();
        
        // Always download the content files
        let content_zip_path = self.temp_dir.join("sinfar_all_files_v30.7z");
        
        // Check if file already exists and is of expected size
        if !self.file_exists_with_size(&content_zip_path, 100_000_000) {
            // File doesn't exist or is too small, download it
            self.download_file(
                "https://sinfar.net/haks/sinfar_all_files_v30.7z", 
                &content_zip_path
            ).await?;
        } else {
            // File exists, update progress to 100%
            let callback = self.progress_callback.lock().await;
            callback(1.0, "sinfar_all_files_v30.7z");
        }
        
        downloaded_files.push(content_zip_path);
        
        // Download version-specific files
        if *game_type == GameType::Diamond {
            let launcher_path = self.temp_dir.join("sinfarx.exe");
            
            if !self.file_exists_with_size(&launcher_path, 10_000) {
                self.download_file(
                    "https://nwn.sinfar.net/files/sinfarx.exe", 
                    &launcher_path
                ).await?;
            } else {
                let callback = self.progress_callback.lock().await;
                callback(1.0, "sinfarx.exe");
            }
            
            downloaded_files.push(launcher_path);
        } else if *game_type == GameType::EnhancedEdition {
            // For EE, download the appropriate launcher for the platform
            #[cfg(windows)]
            {
                let launcher_zip_path = self.temp_dir.join("sinfarLauncher.zip");
                
                if !self.file_exists_with_size(&launcher_zip_path, 10_000) {
                    self.download_file(
                        "https://nwn.sinfar.net/files/sinfarx/8181/win32_8181.zip", 
                        &launcher_zip_path
                    ).await?;
                } else {
                    let callback = self.progress_callback.lock().await;
                    callback(1.0, "sinfarLauncher.zip");
                }
                
                downloaded_files.push(launcher_zip_path);
            }
            
            #[cfg(unix)]
            {
                let launcher_zip_path = self.temp_dir.join("sinfarLauncher.zip");
                
                if !self.file_exists_with_size(&launcher_zip_path, 10_000) {
                    self.download_file(
                        "https://nwn.sinfar.net/files/sinfarx/8181/linux_8181.zip", 
                        &launcher_zip_path
                    ).await?;
                } else {
                    let callback = self.progress_callback.lock().await;
                    callback(1.0, "sinfarLauncher.zip");
                }
                
                downloaded_files.push(launcher_zip_path);
            }
        }
        
        Ok(downloaded_files)
    }
    
    /// Check if a file exists and meets a minimum size requirement
    fn file_exists_with_size(&self, path: &Path, min_size: u64) -> bool {
        if let Ok(meta) = metadata(path) {
            meta.is_file() && meta.len() >= min_size
        } else {
            false
        }
    }
    
    /// Get the path to the temporary directory
    pub fn get_temp_dir(&self) -> &Path {
        &self.temp_dir
    }
}