// src/installer/extractor.rs
use anyhow::{Result, Context};
use sevenz_rust::{Archive, BlockDecoder, decompress};
use std::path::{Path, PathBuf};
use std::sync::Arc;
use tokio::sync::Mutex;
use log::{info, warn, error};
use std::fs;
use std::io::{Read, Write};
use std::collections::HashMap;

pub struct ExtractionManager {
    progress_callback: Arc<Mutex<Box<dyn Fn(f32, &str) + Send + Sync>>>,
}

impl ExtractionManager {
    pub fn new<F>(progress_callback: F) -> Self 
    where 
        F: Fn(f32, &str) + Send + Sync + 'static 
    {
        Self {
            progress_callback: Arc::new(Mutex::new(Box::new(progress_callback))),
        }
    }
    
    /// Extract a 7z archive with progress reporting
    pub async fn extract_7z(&self, archive_path: &Path, output_dir: &Path) -> Result<()> {
        info!("Extracting 7z archive: {} -> {}", archive_path.display(), output_dir.display());
        
        // Make sure the output directory exists
        if !output_dir.exists() {
            tokio::fs::create_dir_all(output_dir).await?;
        }
        
        // Get file size for progress calculation
        let file_size = fs::metadata(archive_path)?.len();
        
        // We'll handle extraction in a blocking task since sevenz-rust is synchronous
        let archive_path = archive_path.to_path_buf();
        let output_dir = output_dir.to_path_buf();
        let progress_callback = self.progress_callback.clone();
        
        tokio::task::spawn_blocking(move || -> Result<()> {
            let archive_file = fs::File::open(&archive_path)?;
            let out = fs::create_dir_all(&output_dir)?;
            
            // decompress takes a Read + Seek and a destination directory
            decompress(&archive_file, &output_dir)?;
        
            // Call progress update with 100% after decompression
            let filename = archive_path.file_name()
                .and_then(|f| f.to_str())
                .unwrap_or("archive");
        
            let progress_callback_clone = progress_callback.clone();
            tokio::runtime::Handle::current().block_on(async move {
                let callback = progress_callback_clone.lock().await;
                callback(1.0, filename); // 100% done
            });
        
            Ok(())
        }).await??;
        
        Ok(())
    }
    
    /// Extract a ZIP archive with progress reporting
    pub async fn extract_zip(&self, archive_path: &Path, output_dir: &Path) -> Result<()> {
        info!("Extracting ZIP archive: {} -> {}", archive_path.display(), output_dir.display());
        
        // Make sure the output directory exists
        if !output_dir.exists() {
            tokio::fs::create_dir_all(output_dir).await?;
        }
        
        // We'll handle extraction in a blocking task since the zip crate is synchronous
        let archive_path = archive_path.to_path_buf();
        let output_dir = output_dir.to_path_buf();
        let progress_callback = self.progress_callback.clone();
        
        tokio::task::spawn_blocking(move || -> Result<()> {
            let file = fs::File::open(&archive_path)?;
            let mut archive = zip::ZipArchive::new(file)?;
            
            let total_entries = archive.len();
            info!("ZIP archive contains {} entries", total_entries);
            
            for i in 0..total_entries {
                let mut file = archive.by_index(i)?;
                let outpath = match file.enclosed_name() {
                    Some(path) => output_dir.join(path),
                    None => continue,
                };
                
                // Update progress
                let progress = (i + 1) as f32 / total_entries as f32;
                let filename = outpath.file_name()
                    .and_then(|f| f.to_str())
                    .unwrap_or("unknown");
                
                if (*file.name()).ends_with('/') {
                    fs::create_dir_all(&outpath)?;
                } else {
                    if let Some(p) = outpath.parent() {
                        if !p.exists() {
                            fs::create_dir_all(p)?;
                        }
                    }
                    let mut outfile = fs::File::create(&outpath)?;
                    std::io::copy(&mut file, &mut outfile)?;
                }
                
                // Fix file permissions on Unix
                #[cfg(unix)]
                {
                    use std::os::unix::fs::PermissionsExt;
                    if let Some(mode) = file.unix_mode() {
                        fs::set_permissions(&outpath, fs::Permissions::from_mode(mode))?;
                    }
                }
                
                // Update progress on the tokio runtime
                let progress_callback_clone = progress_callback.clone();
                tokio::runtime::Handle::current().block_on(async move {
                    let callback = progress_callback_clone.lock().await;
                    callback(progress, filename);
                });
            }
            
            info!("ZIP extraction completed successfully");
            Ok(())
        }).await??;
        
        Ok(())
    }
    
    /// Handles extraction of all downloaded files to their appropriate locations
    pub async fn install_all_files(
        &self, 
        downloaded_files: &[PathBuf], 
        install_path: &Path, 
        game_type: &str, 
        ee_exe_path: Option<&Path>
    ) -> Result<()> {
        // Find and extract the main content files
        let content_archive = downloaded_files.iter()
            .find(|p| p.file_name().map_or(false, |f| f.to_str().unwrap_or("").contains("sinfar_all_files")))
            .context("Content archive not found in downloaded files")?;
            
        // Extract main content
        self.extract_7z(content_archive, install_path).await?;
        
        // Handle game-specific files
        if game_type.to_lowercase() == "diamond" {
            // For Diamond, find and copy the launcher
            let launcher = downloaded_files.iter()
                .find(|p| p.file_name().map_or(false, |f| f.to_str().unwrap_or("").contains("sinfarx.exe")))
                .context("Launcher executable not found in downloaded files")?;
                
            // Copy launcher to install directory
            let target_path = install_path.join("sinfarx.exe");
            tokio::fs::copy(launcher, &target_path).await?;
            
        } else if game_type.to_lowercase() == "enhancededition" || game_type.to_lowercase() == "ee" {
            // For EE, find the launcher zip
            let launcher_zip = downloaded_files.iter()
                .find(|p| p.file_name().map_or(false, |f| f.to_str().unwrap_or("").contains("sinfarLauncher.zip")))
                .context("Launcher ZIP not found in downloaded files")?;
                
            // Extract to the EE executable directory
            if let Some(ee_path) = ee_exe_path {
                // Create the bin/win32_8181 directory if it doesn't exist (Windows)
                // or bin/linux_8181 directory (Linux)
                #[cfg(windows)]
                let extract_dir = ee_path.join("bin").join("win32_8181");
                
                #[cfg(unix)]
                let extract_dir = ee_path.join("bin").join("linux_8181");
                
                // Extract the launcher
                self.extract_zip(launcher_zip, &extract_dir).await?;
            } else {
                return Err(anyhow::anyhow!("EE executable path is required for EE installation"));
            }
        }
        
        info!("All files installed successfully");
        Ok(())
    }
}