pub mod path_selection {
    use eframe::egui::{Ui, TextEdit};
    use crate::app::{SinfarInstallerApp, GameType};
    use crate::installer::validator::*;

    use rfd::FileDialog;
    use std::path::PathBuf;

    pub fn render(ui: &mut Ui, app: &mut SinfarInstallerApp) {
        ui.heading("Select installation paths");
        ui.add_space(10.0);

        // Set default paths if not set
        if app.install_path.is_none() {
            app.install_path = Some(default_install_path(&app.game_type.clone().unwrap_or(GameType::Diamond)));
        }

        if app.game_type == Some(GameType::EnhancedEdition) && app.ee_exe_path.is_none() {
            app.ee_exe_path = Some(default_ee_exe_path());
        }

        ui.label("Please select your Neverwinter Nights install folder:");
        
        ui.horizontal(|ui| {
            let mut path_str = app.install_path.clone().unwrap_or_default().to_string_lossy().to_string();
            let response = ui.add(TextEdit::singleline(&mut path_str).desired_width(ui.available_width() - 80.0));
            if response.changed() {
                app.install_path = Some(PathBuf::from(path_str));
            }
            
            if ui.button("Browse...").clicked() {
                if let Some(path) = FileDialog::new()
                    .set_title("Select Neverwinter Nights Folder")
                    .set_directory(app.install_path.clone().unwrap_or_default())
                    .pick_folder() {
                    app.install_path = Some(path);
                }
            }
        });

        let is_content_path_valid = match &app.game_type {
            Some(GameType::Diamond) => validator::validate_diamond_path(app.install_path.as_ref().unwrap()),
            Some(GameType::EnhancedEdition) => validator::validate_ee_path(app.install_path.as_ref().unwrap()),
            None => false,
        };

        if !is_content_path_valid {
            ui.colored_label(eframe::egui::Color32::RED, "Invalid path: The selected folder must contain both 'tlk' and 'hak' subfolders.");
        }

        // Only show EE path selection if EE is selected
        if app.game_type == Some(GameType::EnhancedEdition) {
            ui.add_space(10.0);
            ui.label("Select the location of the nwn executable for Enhanced Edition:");
            
            ui.horizontal(|ui| {
                let mut ee_path_str = app.ee_exe_path.clone().unwrap_or_default().to_string_lossy().to_string();
                let response = ui.add(TextEdit::singleline(&mut ee_path_str).desired_width(ui.available_width() - 80.0));
                if response.changed() {
                    app.ee_exe_path = Some(PathBuf::from(ee_path_str));
                }
                
                if ui.button("Browse...").clicked() {
                    if let Some(path) = FileDialog::new()
                        .set_title("Select Neverwinter Nights executable location")
                        .set_directory(app.ee_exe_path.clone().unwrap_or_default())
                        .pick_folder() {
                        app.ee_exe_path = Some(path);
                    }
                }
            });
            
            if !validator::validate_ee_exe_path(app.ee_exe_path.as_ref().unwrap()) {
                ui.colored_label(eframe::egui::Color32::RED, "Invalid path: Neverwinter Night EE not detected.");
            }
        }        
    }

    fn default_install_path(game_type: &GameType) -> PathBuf {
        match game_type {
            GameType::Diamond => {
                #[cfg(windows)]
                return PathBuf::from("C:\\Program Files (x86)\\GOG Galaxy\\Games\\NWN Diamond");
                
                #[cfg(unix)]
                return PathBuf::from("/home").join(whoami::username()).join(".local/share/NWN Diamond");
            },
            GameType::EnhancedEdition => {
                #[cfg(windows)]
                return PathBuf::from(dirs::document_dir().unwrap_or_default()).join("Neverwinter Nights");
                
                #[cfg(unix)]
                return PathBuf::from("/home").join(whoami::username()).join(".local/share/Neverwinter Nights");
            },
        }
    }

    fn default_ee_exe_path() -> PathBuf {
        #[cfg(windows)]
        return PathBuf::from("C:\\Program Files (x86)\\GOG Galaxy\\Games\\Neverwinter Nights Enhanced Edition");
        
        #[cfg(unix)]
        return PathBuf::from("/home").join(whoami::username()).join(".local/share/Steam/steamapps/common/Neverwinter Nights");
    }
}